#!/usr/bin/env python
"""
Utility functions for python in unix shell.
"""

import sys,os,time,signal,socket
import threading,unicodedata
from subprocess import Popen,PIPE
from setproctitle import setproctitle

from nosepicker.log import Logger

import argparse

if sys.platform=='darwin':
    CONFIG_PATH = os.path.expanduser('~/Library/Application Support/Systematic')
else:
    CONFIG_PATH = os.path.expanduser('~/.config/systematic')

# Values for TERM environment variable which support setting title
TERM_TITLE_SUPPORTED = [ 'xterm','xterm-debian']

def normalized(path,normalization='NFC'):
    """
    Return given path value as normalized unicode string on OS/X,
    on other platform return the original string as unicode
    """
    if sys.platform != 'darwin':
        return type(path)==unicode and path or unicode(path,'utf-8')
    if not isinstance(path,unicode):
        path = unicode(path,'utf-8')
    return unicodedata.normalize(normalization,path)

def xterm_title(value,max_length=74,bypass_term_check=False):
    """
    Set title in xterm titlebar to given value, clip the title text to
    max_length characters.
    """
    #if not os.isatty(1): return

    TERM=os.getenv('TERM')
    TERM_TITLE_SUPPORTED = [ 'xterm','xterm-debian']
    if not bypass_term_check and TERM not in TERM_TITLE_SUPPORTED:
        return
    sys.stderr.write('\033]2;'+value[:max_length]+'',)
    sys.stderr.flush()

class CommandPathCache(list):
    """
    Class to represent commands on user's search path.
    """
    def __init__(self):
        self.update()

    def update(self):
        """
        Updates the commands available on user's PATH
        """
        paths = []
        self.__delslice__(0,len(self))

        for path in os.getenv('PATH').split(os.pathsep):
            if not paths.count(path):
                paths.append(path)

        for path in paths:
            if not os.path.isdir(path):
                continue
            for cmd in [os.path.join(path,f) for f in os.listdir(path)]:
                if os.path.isdir(cmd) or not os.access(cmd,os.X_OK):
                    continue
                self.append(cmd)

    def versions(self,name):
        """
        Returns all commands with given name on path, ordered by PATH search
        order.
        """
        if not len(self):
            self.update()
        return filter(lambda x: os.path.basename(x) == name, self)

    def which(self,name):
        """
        Return first matching path to command given with name, or None if
        command is not on path
        """
        if not len(self):
            self.update()
        try:
            return filter(lambda x: os.path.basename(x) == name, self)[0]
        except IndexError:
            return None

class ScriptError(Exception):
    """
    Exceptions raised by running scripts
    """
    def __str__(self):
        return self.args[0]

class ScriptThread(threading.Thread):
    """
    Common script thread base class
    """
    def __init__(self,name):
        threading.Thread.__init__(self)
        self.log = Logger(name).default_stream
        self.status = 'not running'
        self.setDaemon(True)
        self.setName(name)
        self._stop_event = threading.Event()

    def stop(self):
        self._stop_event.set()

    @property
    def stopped(self):
        return self._stop_event.isSet()

    def execute(self,command):
        p = subprocess.Popen(command,stdin=sys.stdin,stdout=sys.stdout,stderr=sys.stderr)
        return p.wait()

class Script(object):
    """
    Class for common CLI tool script
    """
    def __init__(self,name=None,description=None,epilog=None,debug_flag=True,subcommands=False):
        self.name = os.path.basename(sys.argv[0])
        setproctitle('%s %s' % (self.name,' '.join(sys.argv[1:])))
        signal.signal(signal.SIGINT, self.SIGINT)

        reload(sys)
        sys.setdefaultencoding('utf-8')

        if name is None:
            name = self.name

        # Set to True to avoid any messages from self.message to be printed
        self.silent = False

        self.logger = Logger(self.name)
        self.log = self.logger.default_stream

        self.parser = argparse.ArgumentParser(
            prog=name,
            description=description,
            epilog=epilog,
            add_help=True,
            conflict_handler='resolve',
        )
        if debug_flag:
            self.parser.add_argument('--debug',action='store_true',help='Show debug messages')

        if subcommands:
            self.commands = {}
            self.command_parsers = self.parser.add_subparsers(
                dest='command', help='Please select one command mode below',
                title='Command modes'
            )

    def SIGINT(self,signum,frame):
        """
        Parse SIGINT signal by quitting the program cleanly with exit code 1
        """
        for t in filter(lambda t: t.name!='MainThread', threading.enumerate()):
            t.stop()
        for t in filter(lambda t: t.name!='MainThread', threading.enumerate()):
            t.join()
        self.exit(1)

    def wait(self,poll_interval=1):
        """
        Wait for running threads to finish.
        Poll interval is time to wait between checks for threads
        """
        while True:
            active = filter(lambda t: t.name!='MainThread', threading.enumerate())
            if not len(active):
                break
            self.log.debug('Waiting for %d threads' % len(active))
            time.sleep(poll_interval)

    def exit(self,value=0,message=None):
        """
        Exit the script with given exit value.
        If message is not None, it is printed on screen.
        """
        if message is not None:
            self.message(message)
        for t in filter(lambda t: t.name!='MainThread', threading.enumerate()):
            t.stop()
        while True:
            active = filter(lambda t: t.name!='MainThread', threading.enumerate())
            if not len(active):
                break
            time.sleep(1)
        sys.exit(value)

    def message(self,message):
        if self.silent:
            return
        sys.stdout.write('%s\n' % message)

    def error(self,message):
        sys.stderr.write('%s\n' % message)

    def register_subcommand(self,command,name,description,epilog=None):
        if name in self.commands:
            raise MusaScriptError('Duplicate sub command name: %s' % name)
        self.commands[name] = command
        return self.command_parsers.add_parser(name,help=description,description=description,epilog=epilog)

    def usage_error(self,*args,**kwargs):
        return self.parser.error(*args,**kwargs)


    def add_argument(self,*args,**kwargs):
        """
        Shortcut to add argument to main argumentparser instance
        """
        self.parser.add_argument(*args,**kwargs)

    def parse_args(self):
        """
        Call parse_args for parser and check for default logging flags
        """
        args = self.parser.parse_args()

        if hasattr(args,'debug') and getattr(args,'debug'):
            self.logger.set_level('DEBUG')
        elif hasattr(args,'quiet') and getattr(args,'quiet'):
            self.silent = True
        elif hasattr(args,'verbose') and getattr(args,'verbose'):
            self.logger.set_level('INFO')

        return args

    def execute(self,args,dryrun=False):
        """
        Default wrapper to execute given interactive shell command
        """
        if not isinstance(args,list):
            raise ValueError('Execute arguments must be a list')

        if dryrun:
            self.log.debug('would execute: %s' % ' '.join(args))
            return 0

        p = Popen(args,stdin=sys.stdin,stdout=sys.stdout,stderr=sys.stderr)
        p.wait()
        return p.returncode



#!/usr/bin/env python
"""
Read test details from nosetests.xml output file
"""

from lxml import etree as ET
from lxml.builder import E

from nosepicker.log import Logger

class NosePicker(dict):
    def __init__(self,path='noseresults.xml',short_classnames=False):
        self.log = Logger('nosepicker').default_stream
        self.path = path

        self.parser = None
        self.fd = None
        self.loaded = False
        self.__next = None
        self.short_classnames = short_classnames

    def __del__(self):
        """
        Close our XML parser before deleting
        """
        self.__close_parser__()

    def __len__(self):
        if not self.parser and not self.loaded:
            self.load()
        return len(self.keys())

    def __getitem__(self,item):
        if not self.parser and not self.loaded:
            self.load()
        try:
            item = int(item)
            return self[self.indexed[item]]
        except ValueError:
            # not integer, lookup by path
            pass
        except IndexError:
            raise DataSourceError('Invalid library index:' % item)
        return NoseResults.__getitem__(self,normalized(item))

    def __open_parser__(self):
        """
        Open XML parser instance
        """
        if self.fd is not None:
            raise DataSourceError('XML file already open: %s' % self.path)
        self.fd = open(self.path,'r')
        self.parser = ET.iterparse(self.fd,events=('start','end',))
        self.parent = None

    def __close_parser__(self):
        """
        Close XML parser instance, interrupting iterator
        """
        if not hasattr(self,'fd') or self.fd is None:
            return
        self.fd.close()
        self.fd = None
        self.parent = None
        if self.parser is not None:
            del self.parser
            self.parser = None

    def __iter__(self):
        return self

    def load(self):
        return

    def next(self):
        """
        First time iterate over XML file on the fly
        After this, return elements loaded by first round
        """

        if not self.loaded:
            if self.parser is None:
                self.__open_parser__()
        elif self.__next is None:
            self.__next = 0

        try:
            if self.loaded:
                # Return elements loaded in first iteration
                try:
                    entry = self[self.indexed[self.__next]]
                    self.__next += 1
                    return entry
                except IndexError:
                    raise StopIteration
            else:
                # Load and return elements from XML dynamically
                return self.__next_xml_element__()
        except StopIteration:
            self.__close_parser__()
            self.loaded = True
            self.__next = None
            self.indexed = self.keys()
            raise StopIteration

    def __next_xml_element__(self):
        """
        Placeholder for function to parse and insert elements to our
        dictionary and playlists.

        Must be implemented in child classes.
        """
        while True:
            (event,element) = self.parser.next()

            if event not in ['start','end']:
                raise ValueError('Unknown XML parser event: %s' % event)

            if event=='end' and element.tag == 'testsuite':
                self.parent = None

            if event=='start':
                if element.tag == 'testsuite':
                    self.parent = NoseTestSuite(self,element)
                    continue

            if event=='end':
                if element.tag == 'testcase':
                    if not self.parent:
                        raise ValueError('Test case outside suite')
                    testcase = NoseTestCase(self.parent,element)
                    return testcase

    def keys(self):
        """
        Return keys (data sources paths) sorted
        """
        return sorted(dict.keys(self))

    def items(self):
        """
        Return data sources items sorted by self.keys() with self.__getitem__
        """
        return [(k,self[k]) for k in self.keys()]

    def values(self):
        """
        Return data sources items sorted by self.keys() with self.__getitem__
        """
        return [self[k] for k in self.keys()]

class NoseTestSuite(object):
    def __init__(self,parent,node):
        self.log = Logger('nosepicker').default_stream
        self.parent = parent
        self.node = node

    @property
    def short_classnames(self):
        return self.parent.short_classnames

class NoseTestCase(object):
    def __init__(self,suite,node):
        self.log = Logger('nosepicker').default_stream
        self.suite = suite
        self.node = node

        self.skipped = [node for node in self.node.xpath('skipped')]
        self.errors = [NoseTestError(self,node) for node in self.node.xpath('error')]
        self.failures = [NoseTestError(self,node) for node in self.node.xpath('failure')]

        for node in self.node:
            if node.tag not in ('error','failure','skipped'):
                self.log.debug('UNEXPECTED: %s' % ET.tostring(node))
                raise ValueError('Unexpected testcase child element: %s' % node.tag)

    def __hash__(self):
        return '%s %s' % (self.classname,self.name)

    def __repr__(self):
        self.log.debug('%8s %s: %d errors %d failures %d skipped' % (
            self.status,self.name,len(self.errors),len(self.failures),len(self.errors)
        ))
        if not self.short_classnames:
            return '%-7s %-40s %s' % (self.status,self.classname,self.name)
        else:
            return '%-7s %-40s %s' % (self.status,self.testclass,self.name)

    @property
    def short_classnames(self):
        return self.suite.short_classnames

    @property
    def status(self):
        if len(self.failures):
            return 'FAILURE'
        elif len(self.errors):
            return 'ERROR'
        elif len(self.skipped):
            return 'SKIP'
        else:
            return 'OK'

    @property
    def time(self):
        return float(self.node.get('time'))

    @property
    def testclass(self):
        return self.node.get('classname').split('.')[0]

    @property
    def classname(self):
        return self.node.get('classname')

    @property
    def name(self):
        return self.node.get('name')

class NoseTestError(object):
    def __init__(self,testcase,node):
        self.log = Logger('nosepicker').default_stream
        self.testcase = testcase
        self.node = node

    def __repr__(self):
        return ET.tostring(self.node)

    @property
    def type(self):
        return self.node.get('type')

    @property
    def consolelog(self):
        return self.node.get('message')

    @property
    def errors(self):
        msg = self.node.text.split('\n')
        lines = []
        for l in msg:
            if l=='-------------------- >> begin captured stdout << ---------------------':
                break
            lines.append(l)
        return '\n'.join(lines)




import os,logging
import logging.handlers

DEFAULT_LOGFORMAT = '%(module)s %(levelname)s %(message)s'
DEFAULT_TIME_FORMAT = "%Y-%m-%d %H:%M:%S"
DEFAULT_LOGFILEFORMAT = '%(asctime)s %(module)s.%(funcName)s %(message)s'
DEFAULT_LOGSIZE_LIMIT = 2**20
DEFAULT_LOG_BACKUPS = 10

class LoggerError(Exception):
    """
    Exceptions raised by logging configuration
    """
    def __str__(self):
        return self.args[0]

class Logger(object):
    """
    Singleton class for common logging tasks.
    """
    __instances = {}
    def __init__(self,name=None):
        name = name is not None and name or self.__class__.__name__
        if not Logger.__instances.has_key(name):
            Logger.__instances[name] = Logger.LoggerInstance(name)
        self.__dict__['_Logger__instances'] = Logger.__instances
        self.__dict__['name'] = name

    class LoggerInstance(dict):
        """
        Singleton implementation of logging configuration for one program
        """
        def __init__(self,name):
            self.name = name
            self.loglevel = logging.Logger.root.level
            self.register_stream_handler('default_stream')

        def __getattr__(self,attr):
            if attr in self.keys():
                return self[attr]
            raise AttributeError('No such LoggerInstance attribute: %s' % attr)

        def __setattr__(self,attr,value):
            if attr in ['level','loglevel']:
                for logger in self.values():
                    logger.setLevel(value)
                self.__dict__['loglevel'] = value
            else:
                object.__setattr__(self,attr,value)

        def register_stream_handler(self,name,logformat=None,timeformat=None):
            """
            Register a common log stream handler
            """

            if name in self.keys():
                raise LoggerError('Handler name already registered to %s: %s' % (self.name,name))

            if logformat is None:
                logformat = DEFAULT_LOGFORMAT
            if timeformat is None:
                timeformat = DEFAULT_TIME_FORMAT

            for logging_manager in logging.Logger.manager.loggerDict.values():
                if hasattr(logging_manager,'name') and logging_manager.name==name:
                    self[name] = logging.getLogger(name)
                    return

            logger = logging.getLogger(name)
            handler = logging.StreamHandler()
            handler.setFormatter(logging.Formatter(logformat,timeformat))
            logger.addHandler(handler)
            self[name] = logger

        def register_file_handler(self,name,directory,
                         logformat=None,
                         maxBytes=DEFAULT_LOGSIZE_LIMIT,
                         backupCount=DEFAULT_LOG_BACKUPS):
            """
            Register a common log file handler for rotating file based logs
            """
            if name in self.keys():
                raise LoggerError('Handler name already registered to %s: %s' % (self.name,name))

            if logformat is None:
                logformat = DEFAULT_LOGFILEFORMAT

            if name in [l.name for l in logging.Logger.manager.loggerDict.values()]:
                return
            if not os.path.isdir(directory):
                try:
                    os.makedirs(directory)
                except OSError:
                    raise LoggerError('Error creating directory: %s' % directory)

            logger = logging.getLogger(name)
            logfile = os.path.join(directory,'%s.log' % name)
            handler = logging.handlers.RotatingFileHandler(
                filename=logfile,
                mode='a+',
                maxBytes=maxBytes,
                backupCount=backupCount
            )
            handler.setFormatter(logging.Formatter(logformat,self.timeformat))
            logger.addHandler(handler)
            logger.setLevel(self.loglevel)
            self[name] = logger

        @property
        def level(self):
            return self.loglevel

        def set_level(self,value):
            if not hasattr(logging,value):
                raise LoggerError('Invalid logging level: %s' % value)
            level = getattr(logging,value)
            if not isinstance(level,int):
                raise LoggerError('Not integer value: %s (%s)' % (value,type(level)))
            self.loglevel = level

    def __getattr__(self,attr):
        return getattr(self.__instances[self.name],attr)

    def __setattr__(self,attr,value):
        setattr(self.__instances[self.name],attr,value)

    def __getitem__(self,item):
        return self.__instances[self.name][item]

    def __setitem__(self,item,value):
        self.__instances[self.name][item] = value





import os
import glob
from setuptools import setup, find_packages

VERSION='1.0.2'

setup(
    name = 'nosepicker',
    version = VERSION,
    license = 'PSF',
    keywords = 'nose xunit output parser',
    url = 'http://tuohela.net/packages/nosepicker',
    scripts = glob.glob('bin/*'),
    author = 'Ilkka Tuohela',
    author_email = 'hile@iki.fi',
    description = 'Scripts to parse nose xunit XML output files',
    packages = ['nosepicker'] + ['nosepicker.%s'%s for s in find_packages('nosepicker')],
    install_requires = (
        'lxml',
        'setproctitle',
    ),
)

